@@ -5,29 +5,43 @@ from . import dependency
 from . import extra
 from . import error
 from . import script
+from . import recipe
 
 all_test_cases = [
+    task.TaskTypeTest,
     task.TaskTest,
     task.TaskDependencyFileExistsTest,
     task.TaskDependencyFileChangedTest,
     task.TaskForcing,
     task.TaskDependencyLinkTest,
     task.TaskNameTest,
+    task.GetPathTest,
 
     dependency.FileDoesNotExistsDependencyTest,
     dependency.FileChangedDependencyTest,
     dependency.AlwaysRebuildDependencyTest,
     dependency.InnerFileExistsTest,
     dependency.InnerFileChangedTest,
+    dependency.FileDependencyTest,
+    dependency.InnerLinkTest,
 
     extra.TouchTest,
     extra.FindFilesTest,
     extra.RunCmdTest,
+    extra.SignalHandlingTest,
+    extra.ProccessTest,
+    extra.TemplateTest,
 
     error.ErrorsTests,
 
-    script.TaskNameParseTest,
+    script._GetTaskTest,
     script.GraphTest,
+    script.InitRecipe,
+    script.CompareVersion,
+
+    recipe.RecipeTest,
+    recipe.RecipeTypeTest,
+    recipe.DownloadRecipeTest,
 ]
 
 
